/**
 * \file gre/elemsampledc.h
 * \brief GRE ELEMSAMPLE_DC class definitions
 *
 * \if NODOC
 * $Id: elemsampledc.h_v 1.11 2013/02/15 20:51:30 mju Exp $
 *
 * $Log: elemsampledc.h_v $
 * Revision 1.11  2013/02/15 20:51:30  mju
 * Fix dox hdr.
 *
 * Revision 1.10  2008/05/27 15:19:18  mju
 * Add setGC.
 *
 * Revision 1.9  2006/09/15 19:44:08  mju
 * Remove hdr dependencies.
 *
 * Revision 1.8  2005/12/19 18:46:54  dwilliss
 * added optional credentials to a method
 *
 * Revision 1.7  2005/10/14 17:30:06  mju
 * GRE namespace.
 *
 * Revision 1.6  2004/02/13 15:32:09  dwilliss
 * deprecated old mgd
 *
 * Revision 1.5  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/07/30 15:18:16  mju
 * Change back to GRE_ name as requires too many other changes.
 *
 * Revision 1.3  2003/07/30 14:47:37  mju
 * Put class in GRE namespace with typedef for old name.
 * Ignore private section.
 *
 * Revision 1.2  2003/06/18 13:57:05  mju
 * Include more req'd headers.
 *
 * Revision 1.1  2003/06/17 13:39:44  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_ELEMSAMPLEDC_H
#define	INC_GRE_ELEMSAMPLEDC_H

#ifndef	INC_GRE_BASE_H
	#include <gre/base.h>
#endif

#ifndef	INC_MI32_MG2BASE_H
	#include <mi32/mg2base.h>
#endif

#ifndef	INC_MI32_MG2ENUMS_H
	#include <mi32/mg2enums.h>
#endif

#ifndef	INC_RVC_STYLE_H
	#include <rvc/style.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_MAPGEN;
#endif

namespace GRE {
//===================================================================================================================

//! Drawing Context for Element Samples.
class ELEMSAMPLE_DC {
	public:

		//! Constructor.
		explicit ELEMSAMPLE_DC (
			MGD::STYLESAMPLEFLAGS SampleFlags = MGD::STYLESAMPLE_ShrinkToFit
			);

		//! Construct with MGC_GC.
		explicit ELEMSAMPLE_DC (
			MGD::CONTEXT *gc,
			MGD::STYLESAMPLEFLAGS SampleFlags = MGD::STYLESAMPLE_ShrinkToFit
			);

		//! Destructor.
		virtual ~ELEMSAMPLE_DC (
			);

		//! Close style object if was opened.
		void CloseStyleObject (
			) { m_StyleObj.Close(); }

		//! Get graphic drawing context.
		MGD::CONTEXT* GetGC (
			) const { return (m_gc); }

		//! Get flags for Draw...StyleSample() usage.
		MGD::STYLESAMPLEFLAGS GetSampleFlags (
			) const { return (m_SampleFlags); }

		//! Get reference to style object.
		const RVC::STYLE& GetStyleObject (
			) const { return (m_StyleObj); }

		//! Get the TransGen used to draw this sample
		//! Will be NULL if drawing into legend view, 
		//! Will be valid if drawing for legend layer.
		const TRANS2D_MAPGEN* GetTransGen (
			) const { return (m_TransGen); }

		//! Open specified style object.
		//! If specified path/inode matches currently open object it will not be opened again.
		ERRVALUE OpenStyleObject (
			const RVC::OBJITEM& StyleObj,
			const RVC::CREDENTIALS* Credentials = 0
			) { return (m_StyleObj.Open(StyleObj, RVC::OBJECT::OPENMODE_Read, 0, Credentials)); }

		//! Set drawing context.
		void SetGC (
			MGD::CONTEXT *gc
			) { m_gc = gc; }

		//! Set flags for Draw...StyleSample() usage.
		void SetSampleFlags (
			MGD::STYLESAMPLEFLAGS SampleFlags
			) { m_SampleFlags = SampleFlags; return; }

		void SetTransGen (
			const TRANS2D_MAPGEN* trans
			) { m_TransGen = trans; return; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MGD::CONTEXT *m_gc;
		MGD::STYLESAMPLEFLAGS m_SampleFlags;
		RVC::STYLE m_StyleObj;
		const TRANS2D_MAPGEN* m_TransGen;

		ELEMSAMPLE_DC (const ELEMSAMPLE_DC&);
		ELEMSAMPLE_DC& operator= (const ELEMSAMPLE_DC&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	//!< INC_GRE_ELEMSAMPLEDC_H
