/**
 * \file gre/scriptcontext.h
 * \brief GRE_SCRIPTCONTEXT classes
 *
 * \if NODOC
 * $Id: scriptcontext.h_v 1.12 2013/02/15 20:49:45 mju Exp $
 *
 * $Log: scriptcontext.h_v $
 * Revision 1.12  2013/02/15 20:49:45  mju
 * Fix dox hdr.
 *
 * Revision 1.11  2009/03/24 21:31:40  mju
 * Use 3 variants when generate style table from script so can specify base style.
 *
 * Revision 1.10  2009/02/13 14:40:59  mju
 * Add generateStyleTableFromScript.
 *
 * Revision 1.9  2009/02/12 17:58:36  mju
 * Add setScaleValues.
 *
 * Revision 1.8  2006/09/28 13:23:39  mju
 * Use lablstyl hdr.
 *
 * Revision 1.7  2006/09/18 20:33:46  mju
 * Use rect/elemtype hdrs.
 *
 * Revision 1.6  2006/09/18 14:16:50  mju
 * Use styletype enum to simplify dependency.
 *
 * Revision 1.5  2006/09/14 17:05:42  mju
 * Use smlbase hdr.
 *
 * Revision 1.4  2006/08/28 14:08:42  mju
 * Use context.release instead of destroy.
 *
 * Revision 1.3  2005/10/19 17:21:16  mju
 * Use gre namespace.
 *
 * Revision 1.2  2005/03/31 16:57:07  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.1  2005/01/20 16:58:21  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_SCRIPTCONTEXT_H
#define	INC_GRE_SCRIPTCONTEXT_H

#ifndef	INC_GRE_BASE
	#include <gre/base.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_LABLSTYL_H
	#include <mi32/lablstyl.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_MG2BASE_H
	#include <mi32/mg2base.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class GEOMETRIC;
	class CADELEMENT;
	class STYLE;
	class OBJITEM;
	}
namespace MGD {
	class CONTEXT;
	}
class TRANS2D_MAPGEN;
#endif

namespace GRE {
//===================================================================================================================

class SCRIPTCONTEXT_DRAW {
	public:

		//! Constructor.
		SCRIPTCONTEXT_DRAW (
			);

		//! Destructor.
		virtual ~SCRIPTCONTEXT_DRAW (
			);

		//! Attach SML context.
		//! After attachment the SCRIPTCONTEXT_DRAW 'owns' the context and will destroy it when
		//! necessary.  The context pointer is passed by reference and set to NULL on return to
		//! prevent external use of the context.
		void AttachContext (
			SMLCONTEXT *& context					//!< Context to attach, will set to NULL on return
			);

		//! Create SML context using RVC::GEOMETRIC object.
		ERRVALUE CreateContext (
			RVC::GEOMETRIC& GeoObj					//!< Geometric object (VECTOR, TIN, CAD, etc)
			);

		//! Destroy attached SML context if any.
		void DestroyContext (
			);

		//! Get attached SMLCONTEXT pointer.
		//! Do not free or Destroy() the returned pointer.
		SMLCONTEXT * GetContext (
			) const { return (m_context); }

		//! Determine if have SMLCONTEXT attached.
		bool HasContext (
			) const { return (m_context != 0); }

		//! Initialize context variables for drawing.
		ERRVALUE InitDraw (
			GRE::LAYER *layer,							//!< Layer
			MGD::CONTEXT *gc,							//!< Drawing device context
			const TRANS2D_MAPGEN& trans,
			bool Is3D = false							//!< 3D setting for CartoScript
			);

		//! Initialize using layer drawing context.
		ERRVALUE InitDraw (
			GRE::LAYERDC& layerdc
			);

		//! Set scaling values.
		void SetScaleValues (
			double DesignScale,
			double CurrentScale
			);

	protected:

		virtual void CreateSymbols (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! SML symbol pointers for standard symbols.
		struct DRAWSYMS {
			SMLSYMBOL *CurMapScale;			//!< Current Mapscale at time of drawing
			SMLSYMBOL *LayoutMapScale;		//!< Layout Mapscale at time of drawing
			SMLSYMBOL *DesignScale;			//!< Design scale at time of drawing
			};

		SMLCONTEXT *m_context;
		DRAWSYMS m_drawsym;

		SCRIPTCONTEXT_DRAW (const SCRIPTCONTEXT_DRAW&);
		SCRIPTCONTEXT_DRAW& operator= (const SCRIPTCONTEXT_DRAW&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

class SCRIPTCONTEXT_STYLE : public SCRIPTCONTEXT_DRAW {
	public:

		//! Constructor.
		SCRIPTCONTEXT_STYLE (
			);

		//! Destructor.
		virtual ~SCRIPTCONTEXT_STYLE (
			);

		//! Create script segment for element and parse specified script.
		//! @return SML 'Segment' for parsed script or error < 0.
		int CreateSegment (
			ELEMTYPE elemtype,						//!< Element type
			const MIUNICODE *scriptstr				//!< Script string
			);

		ERRVALUE Evaluate (
			int segment,								//!< SML segment
			INT32 ElemNum,								//!< Element number, type determined from 'segment'
			const RVC::STYLE& StyleObj,			//!< RVC Style object
			POINTSTYLE& style,						//!< Style passed/returned
			const DPOINT2D* pt = 0,					//!< Sample point for style sample
			bool bDrawingLegendView	= false		//!< true if drawing sample for legend view
			) const;

		ERRVALUE Evaluate (
			int segment,								//!< SML segment
			INT32 ElemNum,								//!< Element number, type determined from 'segment'
			const RVC::STYLE& StyleObj,			//!< RVC Style object
			LINESTYLE& style,							//!< Style passed/returned
			const SIMPLE_ARRAY<DPOINT2D>* pt = 0,		//!< Sample points for style sample
			bool bDrawingLegendView	= false		//!< true if drawing sample for legend view
			) const;

		ERRVALUE Evaluate (
			int segment,								//!< SML segment
			INT32 ElemNum,								//!< Element number, type determined from 'segment'
			const RVC::STYLE& StyleObj,			//!< RVC Style object
			POLYSTYLE& style,							//!< Style passed/returned
			const SIMPLE_ARRAY<DPOINT2D>* pt = 0,		//!< Sample points for style sample
			bool bDrawingLegendView	= false		//!< true if drawing sample for legend view
			) const;

		ERRVALUE Evaluate (
			int segment,								//!< SML segment
			RVC::CADELEMENT& Element,			//!< CAD element
			const RVC::STYLE& StyleObj,			//!< RVC Style object
			const SIMPLE_ARRAY<DPOINT2D>* pt = 0,		//!< Sample points for style sample
			bool bDrawingLegendView	= false		//!< true if drawing sample for legend view
			) const;

		//! Sets the value of the sample rectangle
		void SetSampleRect (
			const DRECT2D& rect
			) { m_stylesym.rect = rect; }

	protected:

		virtual void CreateSymbols (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! SML symbol pointers for standard symbols.
		struct STYLESYMS {
			SMLSYMBOL *Style;					//!< Style name (string)
			SMLSYMBOL *Symbol;				//!< Point symbol name (string)
			SMLSYMBOL *DrawPatt;				//!< Pattern for line drawing (string)
			SMLSYMBOL *FillPatt;				//!< Pattern for filling (string)
			SMLSYMBOL *DrawColor;			//!< Line or Point color (string)
			SMLSYMBOL *FillColor;			//!< Fill color (string)
			SMLSYMBOL *LineScale;			//!< Line Scaling (number)
			SMLSYMBOL *XScale;				//!< X scaling (number)
			SMLSYMBOL *YScale;				//!< Y scaling (number)
			SMLSYMBOL *Angle;					//!< Rotation angle (number)
			SMLSYMBOL *UseStyle;				//!< Flag to use specified style (number)
			SMLSYMBOL *DrawSymbol;			//!< Flag to use point symbol (number)
			SMLSYMBOL *DrawLinePatt;		//!< Flag to use line pattern for drawing (number)
			SMLSYMBOL *DrawBitmapPatt;		//!< Flag to use bitmap pattern for drawing (line or symbol) (number)
			SMLSYMBOL *FillBitmapPatt;		//!< Flag to use bitmap pattern for filling (number)
			SMLSYMBOL *FillHatchPatt;		//!< Flag to use hatch pattern for filling (number)
			SMLSYMBOL *FillInside;			//!< Flag set to fill inside (number)
			SMLSYMBOL *DrawBorder;			//!< Flag set to draw border (number)
			SMLSYMBOL *MapScale;				//!< Mapscale for relative sizes
			SMLSYMBOL *DrawDone;				//!< Flag shows that drawing was already done by query
			SMLSYMBOL *DrawTransPct;		//!< Transparency percentage for "DrawColor"
			SMLSYMBOL *FillTransPct;		//!< Transparency percentage for "FillColor"
			SMLSYMBOL *DrawingSample;		//!< Drawing Sample
			SMLSYMBOL *DrawingLegendView;		//!< Drawing Sample
			SMLSYMBOL *SampleRect;			//!< Symbol when drawing a style sample
			DRECT2D rect;						//!< The rectangle that SampleRect points to
			};

		STYLESYMS m_stylesym;

		void GetColor (
			SMLSYMBOL*,
			SMLSYMBOL*,
			COLOR& color
			) const;

		SCRIPTCONTEXT_STYLE (const SCRIPTCONTEXT_STYLE&);
		SCRIPTCONTEXT_STYLE& operator= (const SCRIPTCONTEXT_STYLE&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

class SCRIPTCONTEXT_LABEL : public SCRIPTCONTEXT_DRAW {
	public:

		//! Constructor.
		SCRIPTCONTEXT_LABEL (
			);

		//! Destructor.
		virtual ~SCRIPTCONTEXT_LABEL (
			);

		void GetFrameParms (
			LABELFRAMEPARMS& parms
			) const;

		double GetPriority (
			) const;

		void SetFrameParms (
			const LABELFRAMEPARMS& parms
			);

		void SetPriority (
			double priority
			);

	protected:

		virtual void CreateSymbols (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! SML symbol pointers for standard symbols.
		struct LABELSYMS {
			SMLSYMBOL *Priority;				//!< Label priority for optimization
			SMLSYMBOL *Frame;					//!< Label frame parameters
			};

		LABELSYMS m_labelsym;
		LABELFRAMEPARMS* m_FrameParms;

		SCRIPTCONTEXT_LABEL (const SCRIPTCONTEXT_LABEL&);
		SCRIPTCONTEXT_LABEL& operator= (const SCRIPTCONTEXT_LABEL&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Generate style assignment table from script using POINTSTYLE.
ERRVALUE GenerateStyleTableFromScript (
	MDLGPARENT dlgparent,
	const RVC::OBJITEM& ObjItemGeometric,
	const RVC::OBJITEM& ObjItemStyleSrc,
	const POINTSTYLE& BaseStyle,
	ELEMTYPE ElemType,
	const MISTRING& ScriptStr,
	RVC::OBJITEM& ObjItemTable
	);

//! Generate style assignment table from script using LINESTYLE.
ERRVALUE GenerateStyleTableFromScript (
	MDLGPARENT dlgparent,
	const RVC::OBJITEM& ObjItemGeometric,
	const RVC::OBJITEM& ObjItemStyleSrc,
	const LINESTYLE& BaseStyle,
	ELEMTYPE ElemType,
	const MISTRING& ScriptStr,
	RVC::OBJITEM& ObjItemTable
	);

//! Generate style assignment table from script using POLYSTYLE.
ERRVALUE GenerateStyleTableFromScript (
	MDLGPARENT dlgparent,
	const RVC::OBJITEM& ObjItemGeometric,
	const RVC::OBJITEM& ObjItemStyleSrc,
	const POLYSTYLE& BaseStyle,
	ELEMTYPE ElemType,
	const MISTRING& ScriptStr,
	RVC::OBJITEM& ObjItemTable
	);

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_SCRIPTCONTEXT_H
