/**
 * \file mgui/cursorid.h
 * \brief MGUI cursor ID definitions
 *
 * \if NODOC
 * $Id: cursorid.h_v 1.12 2011/11/22 18:59:13 mju Exp $
 *
 * $Log: cursorid.h_v $
 * Revision 1.12  2011/11/22 18:59:13  mju
 * Fix dox dependency test.
 *
 * Revision 1.11  2011/10/04 22:00:29  mju
 * Add cursorid_exchange.
 *
 * Revision 1.10  2008/03/31 20:01:46  scowan
 * Added blank cursor.
 *
 * Revision 1.9  2008/01/10 21:01:13  mju
 * Add arrowInto ids.
 *
 * Revision 1.8  2008/01/07 18:04:31  mju
 * Add cursorid_Circle.
 *
 * Revision 1.7  2004/02/11 14:17:40  mju
 * Add PointHand ID.
 *
 * Revision 1.1  2003/01/08 22:45:13  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_CURSORID_H
#define	INC_MGUI_CURSORID_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace MGUI {

//! Enumeration of cursor IDs.
enum CURSORID {
	CURSORID_NotSet = 							0,		//!< Used to reset to default cursor
	CURSORID_ArrowStd = 							1,		//!< Standard selection arrow
	CURSORID_CrosshairThick =					2,		//!< Thick (double-lined) crosshair
	CURSORID_CrosshairThin =					3,		//!< Thin crosshair
	CURSORID_NoFunction =						4,		//!< Indicates no function
	CURSORID_Help =								5,		//!< Arrow with question mark for context help
	CURSORID_Wait =								6,		//!< Hourglas / watch
	CURSORID_MoveHand =							7,		//!< Move/drag hand
	CURSORID_TextInsert =						8,		//!< Text insertion (I-beam)
	CURSORID_SizeAll =							9,		//!< Sizer (arrows pointing up/down/left/right
	CURSORID_SizeNS =								10,	//!< Double-ended arrow pointing up-down (north-south)
	CURSORID_SizeEW =								11,	//!< Double-ended arrow pointing left-right (east-west)
	CURSORID_SizeNESW =							12,	//!< Double-ended arrow pointing up/right-down/left (northeast-southwest)
	CURSORID_SizeNWSE =							13,	//!< Double-ended arrow pointing up/left-down/right (northwest-southeast)
	CURSORID_PointHand =							14,	//!< Pointing hand
	CURSORID_Circle =								15,	//!< Circle
	CURSORID_ArrowIntoCornerBottomLeft =	16,	//!< Arrow pointing into bottom left corner
	CURSORID_ArrowIntoCornerBottomRight =	17,	//!< Arrow pointing into bottom right corner
	CURSORID_ArrowIntoCornerTopLeft =		18,	//!< Arrow pointing into top left corner
	CURSORID_ArrowIntoCornerTopRight =		19,	//!< Arrow pointing into top right corner
	CURSORID_ArrowIntoSideBottom =			20,	//!< Arrow pointing into bottom side
	CURSORID_ArrowIntoSideLeft =				21,	//!< Arrow pointing into left side
	CURSORID_ArrowIntoSideRight =				22,	//!< Arrow pointing into right side
	CURSORID_ArrowIntoSideTop =				23,	//!< Arrow pointing into top side
	CURSORID_Blank =								24,	//!< A cursor that has no graphic, used when following cursor with a graphic
	CURSORID_Exchange =							25,
//	CURSORID_ArrowLeft =			,				// Arrow pointing left
//	CURSORID_ArrowRight =		,				// Arrow pointing right
//	CURSORID_ArrowUp =			,				// Arrow pointing up
//	CURSORID_ArrowDown =			,				// Arrow pointing down
//	CURSORID_ArrowLeftUp =		,				// Arrow pointing left/up (northwest)
//	CURSORID_ArrowLeftDown =	,				// Arrow pointing left/down (southwest)
//	CURSORID_ArrowRightUp =		,				// Arrow pointing right/up (northeast)
//	CURSORID_ArrowRightDown =	,				// Arrow pointing right/down (southeast)
	// Add new cursors to end.
	CURSORID_COUNT
	};


}	// End namespace MGUI

#endif	// INC_MGUI_CURSORID_H
