/*****************************************************************************
*	\file mi32/mail.h
*	\brief MAIL definitions
*
*	\if NODOC
*	$Id: mail.h_v 1.8 2011/10/26 12:58:27 mju Exp $
*
*	$Log: mail.h_v $
*	Revision 1.8  2011/10/26 12:58:27  mju
*	Remove redundant hdrs and fix dox.
*
*	Revision 1.7  2009/09/14 20:01:50  bcolombe
*	Added SetAuthenticate
*	Added SetOutgoingServer
*
*	Revision 1.2  2007/05/31 17:33:05  drb
*	Fixed header info
*	include mistringlist header
*	\endif
*/

#ifndef INC_MI32_MAIL_H
#define INC_MI32_MAIL_H

#ifndef INC_MI32_FILEPATHLIST_H
	#include <mi32/filepathlist.h>
#endif

#ifndef INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

class MAIL {
	public:
	
#ifdef USE_MGUI
		//!default constructor
		MAIL (); 

		//!constructor that will set up the email fields
		MAIL (
			const MISTRING &recipient,
			const MISTRING &subject,
			const MISTRING &body
			); 

		//!constructor that will set up the email fields
		MAIL (
			const MISTRING &recipient,
			const MISTRING &subject,
			const MISTRING &body,
			const MISTRING &sourcename,
			const MISTRING &sourceaddress
			);

		//!default destructor
		~MAIL();

		//!launches the mail client using the settings already specified
		ERRVALUE OpenMailClient(); 

		//!send the email directly using the smtp format
		ERRVALUE SendMail();
		
		//!sets the recipient of the email, only 1 is allowed
		void SetRecipient (
			MISTRING &recipient
			); 

		//!adds an additional attachment to the email
		//!will not function with some mail clients
		//!warning will be given saying so if failed
		void AddAttachment (
			FILEPATH &attachment,
			MISTRING *attachmentname=0
			); 

		//!sets the subject of the email
		void SetSubject (
			MISTRING &subject
			); 

		//!sets the body of the email
		void SetBody (
			MISTRING &body
			); 

		//!sets the email address of the from field of the email
		void SetSourceAddress(
			MISTRING &sourceaddress
			);

		//!sets the name of the from field of the email
		void SetSourceName (
			MISTRING &sourcename
			);

		void SetOutgoingServer (
			const MISTRING& server,
			INT32 port
			) { m_outgoingserver = server; m_port = port;}

		void SetAuthenticate (
			const MISTRING& username,
			const MISTRING& password
			) { m_username = username; m_password = password;}

#endif //end if use_mgui

		//!will make a string RFC compatible, so it could be sent in an email.
		//!does checks like making sure that a single period is not on a line, and 
		//!makes sure no line is longer than 1000 characters
		static MISTRING MakeRFCSMTPCompatible (
			const MISTRING &string
			);

#ifdef USE_MGUI
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		struct ATTACHMENT {
			FILEPATH filepath;
			MISTRING name;
			};

		INT32 m_port;
		MISTRING m_recipient;
		MISTRING m_subject;
		MISTRING m_body;
		MISTRING m_fromname;
		MISTRING m_fromaddress;
		MISTRING m_username;
		MISTRING m_password;
		MISTRING m_outgoingserver;
		MISTRINGLIST m_dnsnameservers;
		MILIST<ATTACHMENT> m_attachments;

		MISTRING MakeSMTPMessage();

		void AddDefaultDNSServers();
		#endif	// GENERATING_DOXYGEN_OUTPUT
		#endif//endif use mgui
	
	};

#endif
