/**
 * \file rvc/editparm.h
 * \brief interface for the RVC::EDITPARM class.
 *
 * \if NODOC
 * $Id: editparm.h_v 1.11 2007/09/06 17:18:45 scowan Exp $
 *
 * $Log: editparm.h_v $
 * Revision 1.11  2007/09/06 17:18:45  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.10  2005/12/09 17:29:26  scowan
 * Updated to use credentials.
 *
 * Revision 1.9  2005/01/19 23:38:56  scowan
 * Support of descriptor class.
 *
 * Revision 1.8  2003/10/07 17:32:52  scowan
 * Object virtual method changes.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2001/06/15 18:59:44  scowan
 * More const.
 *
 * Revision 1.3  2001/06/15 14:48:43  scowan
 * Added to RVC namespace.
 *
 * Revision 1.2  2000/10/19 17:52:27  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 19:59:08  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::EDITPARM rvc/editparm.h
//!	\brief Manages RVC::OBJTYPE_EditParm objects.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_EditParm
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_EditParm
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Vector
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_CAD
//!		- RVC::OBJTYPE_TIN
//!
//!	Read() and Write() methods inherited from RVCSIMPLE class.  INI Handle from RVC::INITEXT class.
//!	View RVC::SIMPLE and RVC::INITEXT docs for inherited methods

#ifndef INC_RVC_EDITPARM_H
#define INC_RVC_EDITPARM_H

#ifndef INC_RVC_INITEXT_H
	#include <rvc/initext.h>
#endif

namespace RVC {

class EDITPARM : public RVC::INITEXT { 
	public:
		EDITPARM (
			);

		EDITPARM (								//! Copy Constructor
			const RVC::EDITPARM& obj
			);

		virtual ~EDITPARM (
			);

 		RVC::EDITPARM& operator= (
			const RVC::EDITPARM& rhs
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);
		
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_EDITPARM_H
