/**
 * \file rvc/initext.h 
 * \brief interface for the RVC::INITEXT class.
 *
 * \if NODOC
 * $Id: initext.h_v 1.15 2007/09/06 17:18:51 scowan Exp $
 *
 * $Log: initext.h_v $
 * Revision 1.15  2007/09/06 17:18:51  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.14  2005/12/09 17:29:37  scowan
 * Updated to use credentials.
 *
 * Revision 1.13  2005/07/19 17:49:06  scowan
 * Update to use current simple methods.
 *
 * Revision 1.12  2005/07/06 17:17:09  scowan
 * Changed names of some methods due to conflict with base classes.
 *
 * Revision 1.11  2005/07/06 17:13:37  scowan
 * Added get dft namedesc override.
 *
 * Revision 1.10  2005/07/06 16:54:27  scowan
 * Made non-pure class.
 *
 * Revision 1.9  2005/07/06 16:30:53  scowan
 * Made virtual overrides protected.
 *
 * Revision 1.8  2003/10/07 17:32:25  scowan
 * Object virtual method changes.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:56:58  dwilliss
 * doxygen
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2002/04/19 14:38:46  scowan
 * Added attach and detach methods.
 *
 * Revision 1.3  2001/06/14 21:50:30  scowan
 * Added to RVC namespace.
 *
 * Revision 1.2  2000/10/19 17:56:30  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 19:58:51  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::INITEXT rvc/initext.h
//!	\brief Interface for RVC objects that support INI file text in RVC text objects
//!
//!	This includes methods to read from and write to a RVC text object given an Ini Handle
//!
//!	RVC Object Types that this class supports: Derived class
//!
//!	Default RVC Object Type for this class: Derived class
//!
//!	Valid parents for this class: Derived class

#ifndef INC_RVC_INITEXT_H
#define INC_RVC_INITEXT_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

namespace RVC {

class INITEXT : public RVC::SIMPLE {
	public:

		//! Default constructor
		INITEXT (
			);

		//! Copy constructor, not to be used lightly, makes copy of ini string
		INITEXT (
			const RVC::INITEXT& obj
			);

		//! Destructor
		virtual ~INITEXT (
			);

		//! Assignment, not to be used lightly, makes copy of ini string
		RVC::INITEXT& operator= (
			const RVC::INITEXT& rhs
			);

		//! Attach (gain ownership) of an INI handle to this object instance.
		//! Will free handle when object instance goes out of scope
		//! Handle set to NULL on return
		void AttachHandle (
			INIHANDLE& handle
			);

		//! Detach (lose ownership) of an INI handle from this object instance 		
		INIHANDLE DetachHandle (
			);

		//! Obtain INI handle for reading / writing information
		//! @return INIHANDLE
		INIHANDLE GetIniHandle (
			) const;

	protected:			
		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INIHANDLE m_IniHandle;

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_INITEXT_H
