/**
 * \file rvc/opgeometrictoraster.h
 * \brief Operation to convert geometric object to raster by drawing
 *
 * \if NODOC
 * $Id: opgeometrictoraster.h_v 1.4 2011/10/13 21:11:58 mju Exp $
 *
 * $Log: opgeometrictoraster.h_v $
 * Revision 1.4  2011/10/13 21:11:58  mju
 * Remove redundant hdr.
 *
 * Revision 1.3  2011/09/02 12:53:05  mju
 * Dox hdr.
 *
 * Revision 1.2  2007/04/16 19:33:41  scowan
 * Made methods static.
 *
 * Revision 1.1  2007/04/16 19:24:37  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPGEOMETRICTORASTER_H
#define	INC_RVC_OPGEOMETRICTORASTER_H

#ifndef	INC_RVC_GEOMETRC_H
	#include <rvc/geometrc.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_MAPGEN;
namespace RVC {
	class ELEMENT;
	class OBJITEM;
	}
#endif


namespace RVC {
namespace OP {

//=====================================================================================================================

//! Converter for performing raster to geometric conversion by lineary tracing.
class GEOMETRIC_TO_RASTER {
	public:
		//! Convert (render) geometric elements into a raster object.
		static ERRVALUE Process (
			const RVC::OBJITEM& SrcObjItem,
			const RVC::OBJITEM& DestObjItem,					//!< Destination raster objitem (must exist)
			const TRANS2D_MAPGEN& GeoToRast,
			RVC::GEOMETRIC::COMPONENT Component,			//!< COMPONENT_Line, Point, and / or Polygon
			double FillValue
			);
	
		//! Convert (render) a geometric element into a raster object.
		static ERRVALUE Process (
			const RVC::OBJITEM& SrcObjItem,
			const RVC::OBJITEM& DestObjItem,					//!< Destination raster objitem (must exist)
			const TRANS2D_MAPGEN& GeoToRast,
			const RVC::ELEMENT& Element,
			double FillValue
			);
	};

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPGEOMETRICTORASTER_H
