/**
 * \file rvc/uidlgeditmetadata.h
 * \brief RVC::UI::DLG_EDIT_METADATA definitions
 *
 * \if NODOC
 * $Id: uidlgeditmetadata.h_v 1.3 2013/01/24 15:17:16 mju Exp $
 *
 * $Log: uidlgeditmetadata.h_v $
 * Revision 1.3  2013/01/24 15:17:16  mju
 * Use menu_edit.
 *
 * Revision 1.2  2013/01/22 14:29:31  mju
 * Add createAndOpen.
 *
 * Revision 1.1  2013/01/21 18:54:54  mju
 * Initial revision
 *
 * \endit
**/

#ifndef	INC_RVC_UIDLGEDITMETADATA_H
#define	INC_RVC_UIDLGEDITMETADATA_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_MENUEDIT_H
#include <mgui/menuedit.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

namespace RVC {
namespace UI {

//=====================================================================================================================

//! Dialog to allow editing metadata of specified object.
class DLG_EDIT_METADATA : public MGUI::DLGSHELL {
	public:

		//! Create and open modeless dialog.
		static ERRVALUE CreateAndOpen (
			MDLGPARENT dlgparent,					//!< Parent for dialog
			const RVC::OBJITEM& ObjItem			//!< Object to view/edit metadata for
			);

		//! Constructor.
		explicit DLG_EDIT_METADATA (
			const RVC::OBJITEM& ObjItem			//!< Object to view/edit metadata for
			);

		//! Destructor.
		virtual ~DLG_EDIT_METADATA ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_EDIT_METADATA THISCLASS;
		class FILTER_GETOBJECT;

		enum CMDID {
			CMDID_File_Open =		1,
			CMDID_File_Save,
			CMDID_File_SaveAsText,
			CMDID_Edit_InsertFile,
			CMDID_Edit_InsertTextObj,
			CMDID_Close
			};

		MGUI::MENU m_MenuFile;
		MGUI::MENU_EDIT m_MenuEdit;
		MGUI::CTRL_EDIT_TEXT m_EditText;
		RVC::OBJITEM m_ObjItemParent;
		bool m_bAllowEdit;
		bool m_bIsChanged;

		ERRVALUE DoSave (bool bAskIfChanged);
		void OnEditTextModify ();
		void OnMenu (MGUI::ID cmdid, bool ToggleOn);
		ERRVALUE SetupNewObject ();

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnCloseRequest ();
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIDLGEDITMETADATA_H
