/** 
 * \file rvc/uieditrastervalue.h
 * \brief Definitions for RVC::UI::FORM_EDIT_RASTER_VALUE and RVC::UI::DLG_EDIT_RASTER_VALUE
 *
 * \if NODOC
 * $Id: uieditrastervalue.h_v 1.7 2011/12/27 16:46:21 mju Exp $
 *
 * $Log: uieditrastervalue.h_v $
 * Revision 1.7  2011/12/27 16:46:21  mju
 * Remove redundant hdr.
 *
 * Revision 1.6  2011/09/02 12:57:32  mju
 * Dox hdr.
 *
 * Revision 1.5  2008/04/18 22:25:24  scowan
 * Added component labels and a style to allow them.
 *
 * Revision 1.4  2007/12/12 18:23:46  scowan
 * Added orientation parm to create method.
 *
 * Revision 1.3  2007/04/02 17:43:06  scowan
 * Include necessary any raster value include file.
 *
 * Revision 1.2  2006/07/21 21:46:22  mju
 * Add doxy comment.
 *
 * Revision 1.1  2006/06/28 16:27:59  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIEDITRASTERVALUE_H
#define	INC_RVC_UIEDITRASTERVALUE_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_RVC_IMAGE_H
#include <rvc/image.h>
#endif

#ifndef	INC_MI32_ANYRASTV_H
	#include <mi32/anyrastv.h>
#endif

namespace RVC {
namespace UI {

//===================================================================================================================
//! Form consisting of label and edit field(s) for specified image cell type.
class FORM_EDIT_RASTER_VALUE : public MGUI::FORM_COMPOSITE {
	public:
		enum STYLE {
			STYLE_Default = 0x00,
			STYLE_ComponentLabels = 0x01		//!< Preceed the value prompts 
			};
	
		//! Constructor.
		FORM_EDIT_RASTER_VALUE (
			);

		//! Destructor.
		virtual ~FORM_EDIT_RASTER_VALUE (
			);

		//! Clear current value.
		void ClearValue (
			bool notify = true							//!< Send notification that value changed
			);

		//! Create form with label from resource lookup.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label
			RVC::IMAGE::CELLTYPE CellType,
			STYLE Style = STYLE_Default,
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Horizontal,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Retrieve label control for form.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_label); }

		//! Get current raster value.
		void GetValue (
			ANYRASTVALUE& value
			);
			
		//! Determine if value has been set.
		bool HasValue (
			);

		//! Set cell type.			
		void SetCellType (
			RVC::IMAGE::CELLTYPE CellType
			);

		//! Set delegate to call when value changed.
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeValue = delegate; }

		//! Set label text.
		void SetLabel (
			const MISTRING& label					//!< New label text
			) { m_label.SetLabel(label); }

		//! Set current value with validation.
		//! If value specified is not valid for current raster type the value will not be updated.
		//! No comparison is made with the current value.
		void SetValue (
			const ANYRASTVALUE& value,				//!< New value to set
			bool notify = true						//!< Call OnChangeValue() if value actually updated
			);

	protected:

		//! Called when value is changed.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnChangeValue ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeValue;
		RVC::IMAGE::CELLTYPE m_CellType;
		MGUI::CTRL_LABEL m_label;
		MGUI::CTRL_EDIT_NUMBER m_Value1;
		MGUI::CTRL_EDIT_NUMBER m_Value2;
		MGUI::CTRL_EDIT_NUMBER m_Value3;
		MGUI::CTRL_EDIT_NUMBER m_Value4;
		
		//! Used if STYLE_ComponentName is specified
		MGUI::LAYOUT_PANE m_ValuePane1;
		MGUI::CTRL_LABEL m_ValueLabel1Real;
		MGUI::CTRL_LABEL m_ValueLabel1Magnitude;
		MGUI::CTRL_LABEL m_ValueLabel1Red;
		MGUI::CTRL_LABEL m_ValueLabel1Cyan;
		MGUI::LAYOUT_PANE m_ValuePane2;
		MGUI::CTRL_LABEL m_ValueLabel2Imaginary;
		MGUI::CTRL_LABEL m_ValueLabel2Phase;
		MGUI::CTRL_LABEL m_ValueLabel2Green;
		MGUI::CTRL_LABEL m_ValueLabel2Magenta;
		MGUI::LAYOUT_PANE m_ValuePane3;
		MGUI::CTRL_LABEL m_ValueLabel3Blue;
		MGUI::CTRL_LABEL m_ValueLabel3Yellow;
		MGUI::LAYOUT_PANE m_ValuePane4;
		MGUI::CTRL_LABEL m_ValueLabel4Alpha;
		MGUI::CTRL_LABEL m_ValueLabel4Black;
		bool m_IsVertical;
		
		void OnValueChanged ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Dialog allowing editing of raster cell value.
class DLG_EDIT_RASTER_VALUE : public MGUI::DLGSHELL {
	public:
		//! Constructor.
		DLG_EDIT_RASTER_VALUE (
			const MISTRING& title,						//!< Title string, can use TEXTID also
			const MISTRING& label,						//!< Label
			RVC::IMAGE::CELLTYPE CellType,
			HELPID HelpID = HELPID__None				//!< Help ID
			);

		//! Destructor.
		virtual ~DLG_EDIT_RASTER_VALUE (
			);

		//! Get current raster value.
		void GetValue (
			ANYRASTVALUE& value
			);
			
		//! Determine if value has been set.
		bool HasValue (
			);
			
		//! Set delegate to call when value changed.
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set current value with validation.
		//! If value specified is not valid for current raster type the value will not be updated.
		//! No comparison is made with the current value.
		void SetValue (
			const ANYRASTVALUE& value,				//!< New value to set
			bool notify = true						//!< Call OnChangeValue() if value actually updated
			);

	private:
		FORM_EDIT_RASTER_VALUE m_FormEditRasterValue;
		RVC::IMAGE::CELLTYPE m_CellType;
		MISTRING m_label;
		ANYRASTVALUE m_value;
		
		virtual ERRVALUE v_CreateContent ();
	};

//===================================================================================================================

}	// End of UI namespace
}	// End of RVC namespace

#endif	// INC_RVC_UIEDITRASTERVALUE_H
