/******************************************************************************
 *
 * \file sml/fileinfo.h
 * \brief SML FILEINFO
 *
 * \if NODOC
 * $Log: fileinfo.h_v $
 * Revision 1.3  2011/10/13 21:13:03  mju
 * Remove redundant hdr.
 *
 * Revision 1.2  2007/05/08 15:45:46  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 21:53:48  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_FILEINFO_H
#define INC_SML_FILEINFO_H

#ifndef	INC_MI32_MICALLBACKLIST_H
	#include <mi32/micallbacklist.h>
#endif
#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif
#ifndef WIN32_NATIVE
#include <X11/Intrinsic.h>
#endif

namespace SML {

class CLASS;
class CONTEXT;

/***************************************************************************/
/*
**		An SML class FILE handle points to one of these.  This is here so that
**		the file, port, and console routines can be in separate files.
**
**		Some day, this will all be rewritten as a base class and derived
**		classes with virtual functions instead of all these function pointers.
*/
struct FILEINFO {

	FILEINFO () :
		handle(0),
		ThisClass(0),
		blocking(0),
		isfile(0),
		prevchar(0),
#ifndef WIN32_NATIVE
		m_timer(0),			//!< Timer for polling
#endif
		m_PollInterval(0),
		m_conv(0)
		{ }

	virtual ~FILEINFO () { }

	void *handle;
	CLASS *ThisClass;		//!<  Ick, emulate RTTI
	int blocking;		//!<  Wait for input
	int isfile;			//!<  This is some sort of file

	void Close (
		) { v_Close(); }

	INT32 Read (
		UINT8* buf, 
		INT32 BytesToRead, 
		CONTEXT* context
		) { return (v_Read(buf, BytesToRead, context)); } 

	INT32 Write (
		const UINT8* buf, 
		INT32 BytesToWrite, 
		CONTEXT* context
		) { return (v_Write(buf, BytesToWrite, context)); } 

	INT32 ReadASCIINum (
		double* pVal, 
		CONTEXT* context
		) { return (v_ReadASCIINum(pVal, context)); }

	bool IsEOF (
		) { return (v_IsEOF()); }

	bool IsOpen (
		) const { return (v_IsOpen()); }

	INT32 Seek (
		INT32 offset,
		int mode
		) { return (v_Seek(offset, mode)); }


	void Printf(CONTEXT* context, const char* format, ...);

	char prevchar;	//!<  For detecting <CR-LF>
	MICALLBACKLIST m_cblist;
	INT32 m_PollInterval;		//!< Polling interval in ms
	void* m_conv;
#ifndef WIN32_NATIVE
	XtIntervalId m_timer;			//!< Timer for polling
#endif
	private:
		virtual void v_Close();
		virtual INT32 v_Read(UINT8* buf, INT32 BytesToRead, CONTEXT* context);
		virtual INT32 v_Write(const UINT8* buf, INT32 BytesToWrite, CONTEXT* context);
		virtual INT32 v_ReadASCIINum(double* pVal, CONTEXT* context);
		virtual bool v_IsEOF();
		virtual bool v_IsOpen() const;
		virtual INT32 v_Seek(INT32, int);
	};

} // end of SML namespace

typedef SML::FILEINFO SMLFILEINFO;

#endif		//  INC_SML_FILEINFO_H


