/******************************************************************************
 *
 * \file sml/symbol.h
 * \brief SML SYMBOL
 *
 * \if NODOC
 * $Log: symbol.h_v $
 * Revision 1.8  2011/10/12 14:32:51  mju
 * Remove redundant hdrs.
 *
 * Revision 1.7  2008/10/31 16:13:50  scowan
 * Removed sml_sameSymbol define.
 *
 * Revision 1.6  2008/10/23 20:38:18  scowan
 * Moved function prototypes from sml include file.
 *
 * Revision 1.5  2007/05/08 15:46:29  dwilliss
 * SML namespace
 *
 * Revision 1.4  2007/05/07 15:43:40  dwilliss
 * *** empty log message ***
 *
 * Revision 1.3  2007/05/04 22:46:57  dwilliss
 * *** empty log message ***
 *
 * Revision 1.2  2007/05/04 21:32:59  dwilliss
 * *** empty log message ***
 *
 * Revision 1.1  2007/05/04 21:28:08  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_SYMBOL_H
#define INC_SML_SYMBOL_H

#ifndef	INC_SML_USERDATA_H
#include <sml/userdata.h>
#endif

#ifndef	INC_SML_STYPE_H
#include <sml/stype.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
#include <mi32/smlbase.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

class MISTRING;


#ifdef GetObject
//! If we've included windows.h before this, they #defined GetObject
//! for us.  We don't want it defined!
#undef GetObject
#endif

namespace SML {

#ifndef GENERATING_DOXYGEN_OUTPUT
typedef PTRINT MACHINEINST;	//!<  not (INT32),  Must be same size as (void *)
class STRING;
class CLASSARRAY;
class USERDATA;
class RASTEROBJECT;
class VECTOROBJECT;
class CADOBJECT;
class TINOBJECT;
class SHAPEOBJECT;
class OBJECT;
class FUNC;
class PARM;
class TABLEDATA;
class CONTEXT;
class CLASS;
#endif

/* Flags in SML::SYMBOL.flags (only a short) */

#define SYMBOL_DontFree		0x0100	//!<  Thing points to something that shouldn't be free'd
#define SYMBOL_Permanent	0x0200	//!<  Don't dispose of contents
#define SYMBOL_Static		0x0400	//!<  Was declared as static


/***************************************************************************/
//! Class for representing variables and functions in SML.
//!
//! Note:  This class must not have virtual functions and the members
//! 	need to stay the same size and in the same order.  Any new
//! 	members must be added to the end.
//!
// Members labeled PRIVATE are not accessed outside the DLL as of 6.4
// Members labeled PUBLIC are accessed outside the DLL

class RVCSYSLIBCLASSEXPORT SYMBOL {
public:

	enum SCOPE {
		SCOPE_NONE			= 0,
		SCOPE_TABLE,
		SCOPE_BFUNCTION,
		SCOPE_KEYWORD,
		SCOPE_GLOBAL,
		SCOPE_LOCAL,
		SCOPE_PERMANENT,
		SCOPE_PROTOTYPE,
		SCOPE_STATIC
		};

public:

	// public member functions
	//! Attach some User Data to the symbol.
	//! The data to be attached must be derived from SML::USERDATA, and must have
	//!	been allocated via "new".  Once attached to the symbol, the symbol owns the
	//! data and will "delete" it when the symbol is deleted.
	//! Also, if you try to attach something for a user ID that's already in use,
	//! it will delete the existing thing.
	void AttachUserData (
		USERDATA* data
		) {
		m_userdata.Attach(data);
		return;
		}

	//! Unattach some User Data from the symbol.
	//! If the data to be unattached isn't attached, it just ignores you.
	//! Once unattached, the data is yours to do with as you wish.  You are
	//! responsible for deleting it too.
	void DetachUserData (
		USERDATA* data
		) {
		m_userdata.Detach(data);
		return;
		}

	//! Get the User Data attached to a symbol
	void DisposeOfContents (
		);

	double* GetArray (
		) const;

	int GetArrayNumLins (
		) const;

	int GetArrayNumCols (
		) const;

	//! Returns the class pointer if the symbol is a CLASS
	const CLASS* GetClass (
		) const;

	//! If no data for the given id is attached, it will just return NULL	.
	//! Do not free the data returned unless you Unattach it first.
	USERDATA* GetUserData (
		USERDATA::TAG tag
		) {
		return (m_userdata.Get(tag));
		}

	// Return true if the symbol is a "permanent" symbol defined by the
	// creator of the script context (e.g. defined by a style query)
	bool IsPermanent (
		) const;

	//! Set string symbol value (MIUNICODE*)
	ERRVALUE SetValue (
		const MIUNICODE *string				//!< Value to set
		);

	void SetValuePtr (
		void* val
		);

	//! Return the DBFIELDTYPE of the field (if the symbol refers to a field).
	//!
	//! Note, the return value may be undefined gibberish if the symbol does
	//! not refer to an STYPE_StrField or STYPE_NumField
	int GetFieldType (
		) const;

	//! Return the SML::SYMBOL of the Nth argument of a function.
	//!
	//! If the symbol is for a built-in function, this method will
	//! return the symbol of the Nth argument at runtime.
	SYMBOL* GetFormalParameterSymbol (
		int parmnum
		) const;

	//! Return the SML::CLASS of the Nth argument of a function.
	//!
	//! If the symbol is for a built-in function, this method will
	//! return the SML::CLASS of the Nth parameter, or NULL if the Nth
	//! parameter isn't a class
	CLASS* GetFormalParameterClass (
		int parmnum
		) const;

	//! Return the type of the Nth argument of a function.
	int GetFormalParameterType (
		int parmnum
		) const;

	//! Get the symbol name.
	const MIUNICODE* GetName (
		) const;

	//! Get the alternative symbol name.
	//! This is used for symbols pointing to RVC tables and fields which may
	//!	have both long names and short names.  In that case, the "Alt" name
	//! 	is the long name.  If there is no alternative name, this just returns
	//!	the normal name.
	const MIUNICODE* GetAltName (
		) const;

	//! Get the next symbol in the linked list.
	SYMBOL* GetNext (
		);

	//! Get the next symbol in the linked list (const).
	const SYMBOL* GetNext (
		) const;

	int GetObject (
		OBJECT** pObj,
		bool UndefinedOK = false
		);

	int GetObject (
		RASTEROBJECT** pObj,
		bool UndefinedOK = false
		);

	int GetObject (
		VECTOROBJECT** pObj,
		bool UndefinedOK = false
		);

	int GetObject (
		CADOBJECT** pObj,
		bool UndefinedOK = false
		);

	int GetObject (
		TINOBJECT** pObj,
		bool UndefinedOK = false
		);

	//! Get the field number for this symbol.
	//! This only applies to symbols of STYPE_StrField or STYPE_NumField
	int GetFieldNumber (
		) const;

	//! Get the SML::TABLEDATA for this symbol.
	//! This only applies to symbols of STYPE_Table, STYPE_StrField or STYPE_NumField
	//! Will return NULL for other types of symbols.
	const TABLEDATA* GetTableData (
		) const;

	//! Get the table number for this symbol.
	//! This only applies to symbols of STYPE_Table, STYPE_StrField or STYPE_NumField
	int GetTableNumber (
		) const;

	STYPE GetType (
		) const;

	void SetType (
		STYPE type
		);

	const char *GetTypeName (
		) const;

	//! Returns true if the symbol represents a NULL value.
	int IsNULL (
		) const;

	//! Returns true if the symbol is an optional function argument.
	bool IsOptionalArgument (
		) const;


	void SetObjectElement (
		int ElemType,
		INT32 ElemNum
		);

	void SetRasterLinCol (
		int lin,
		int col
		);


	//! Get value as 'double'.
	//! @return true if symbol has a falue, false if not
	//! leaves value unchanged if symbol does not have a value
	bool GetValue (
		double& value
		) const;

	//! Get value as MISTRING.
	//! @return true if symbol has a falue, false if not
	//! leaves value unchanged if symbol does not have a value
	bool GetValue (
		MISTRING& value
		) const;

	MIUNICODE *GetValueUnicode (
		char *fmt,
		MIUNICODE *buf,
		int maxlen
		) const ;

	char *GetValueStr (
		char *fmt,
		char *buf,
		int maxlen
		) const;

#if defined(ALLOW_RETIRED) || defined (RVCSYSDLL)
	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Get the value of the symbol into an MISTRING.
	//!
	//! format parameter is only used if the symbol is holding a numeric value.
	//! RETIRED -- Use the version that doesn't take a format
	void GetValue (
		MISTRING& mistring,
		const char* format
		) const;
	#endif // GENERATING_DOXYGEN_OUTPUT
#endif

	double GetValueDouble (
		) const;

	void* GetValuePtr () const;
	OBJECT* GetValueObj () const;

	//! Set the value of the symbol to the given object.
	//! Only do this if the symbol is already an object symbol
	void SetValue(OBJECT*);

	STRING* GetValueSTRING() const;

	ERRVALUE ResizeArray (
		int numcols
		);

	ERRVALUE ResizeArray (
		int numlines,
		int numcols,
		bool bKeep = false
		);

	void SetArray (
		double *data,
		int numrows,
		int numcols
		);

	//! Set the value to IEEE_NaN
	int SetValueNULL (
		);

	//! Set numeric symbol value (double)
	int SetValue (
		double value
		);

	//! Set string symbol value (char*)
	ERRVALUE SetValue (
		const char *string					//!< Value to set
		);


#ifndef GENERATING_DOXYGEN_OUTPUT
//!The following functions are for internal use only.
	//! DONT'T USE THIS except qqvect.c
	SYMBOL* GetLocalVars (
		) const;

	//! DONT'T USE THIS except qqrast.c
	void SetLocalVars (
		SYMBOL* sym
		);

	//! DONT'T USE THIS except qqrast.c
	void SetNext (
		SYMBOL* sym
		);

	void _SetNumCols(int);

	//! DONT'T USE THIS except smldocs.c
	FUNC* GetFunc() const;

	int GetTypeRaw() const;

	// LIST OF FRIENDS
	friend class CONTEXT;
	friend class VM;
	friend class PARSER;
	friend int qqparse(void*);
#endif // GENERATING_DOXYGEN_OUTPUT

private:
	void SetValuePtr (
		OBJECT* val
		);


#ifndef GENERATING_DOXYGEN_OUTPUT
public:
#if !defined(RVCSYSDLL)
private:
#endif

	/*:> public */
	MIUNICODE *name;

	union DATA {		/* 12 bytes */
		double			val;
		void *			ptr;
		void				(*proc)(int, ARG*, UINT8 *, CONTEXT *);			/* hardcoded functions */
		double			(*func)(int, ARG*, UINT8 *, CONTEXT *);			/* hardcoded functions */
		double			(*dfunc)(double);			/* hardcoded functions */
		SYMBOL		*(*ofunc)(int, ARG*, UINT8 *, CONTEXT *);		/* hardcoded functions */
		STRING *		(*sfunc)(int, ARG*, UINT8 *, CONTEXT *);			/* hardcoded functions */
		void *			(*classfunc)(int, ARG*, UINT8 *, CONTEXT *);	/* hardcoded functions of type STYPE_ClassRefFunc or STYPE_ClassCopyFunc */
		MACHINEINST	defn;
		void 				*list;
		STRING		*str;
		double 			*array;
		UINT8				*free_me;
		int 				pc;					//!<  for LABELS. -1 == undefined
		void				*ClassData;
		CLASSARRAY  *carray;
		struct {
			short tnum, fnum, ftype, spare;	//!<  spare is to fix alignment
			} field;
		} u;
	short type;
	short ptype;	//!<  NOT STYPE!
	PARM *parm;	//!<  redundant -- same as smlfunc->parms
	union {
		short parmnum;					//!<  parameter number if non-zero
		short	segnum;					//!<  for userdefined functions
		} u1;
	short flags;						//!<  PARM_Optional, PARM_ByRef, etc

	long NumLins;
	long NumCols;			//!<  Dimensions for array
	union {
		TABLEDATA *table;			//!< For table and field symbols only
		CLASS *objclass;			//!< For class symbols
		} u2;
	SYMBOL *tablesym;		//!< If this is a field symbol, tablesym points back to the parent table 
	SYMBOL *localvars;		//!<  local variables or chain of fields w/ same name
	SYMBOL *next;
	FUNC *smlfunc;
	CONTEXT *context;
	USERDATACONTAINER m_userdata;
	OBJECT* m_obj;
	MIUNICODE* m_altname;		// Alternative name ("full name" for objects)
	SYMBOL* m_NextOverload;
	INT32 m_Scope;					//	for "local" symbols 

	#endif // GENERATING_DOXYGEN_OUTPUT

	};

//! Returns true if two symbols refer to the same RVC object
RVCSYSLIBEXPORT int SameObject (
	SML::SYMBOL*,
	SML::SYMBOL*
	);
}			// End of SML namespace

/***************************************************************************/

typedef enum SML::SYMBOL::SCOPE SMLSCOPE;
typedef SML::SYMBOL SMLSYMBOL;

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_SYMBOL_H


