/**
 * \file spatmod/imagefiltergapfill.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_GAPFILL definition.
 *
 * \if NODOC
 * $Id: imagefiltergapfill.h_v 1.4 2010/07/22 16:54:47 mju Exp $
 *
 * $Log: imagefiltergapfill.h_v $
 * Revision 1.4  2010/07/22 16:54:47  mju
 * Update for changes to filling algorithm.
 *
 * Revision 1.3  2009/12/23 22:58:54  mju
 * Add method_none.
 *
 * Revision 1.2  2009/12/23 20:49:44  mju
 * Rename as filter_gapfill.
 *
 * Revision 1.1  2009/12/23 20:40:06  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERGAPFILL_H
#define	INC_SPATMOD_IMAGEFILTERGAPFILL_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_BITSET_H
#include <mi32/bitset.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#include <vector>

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to fill gaps in image.
class FILTER_GAPFILL : public FILTER {
	public:

		enum METHOD {
			METHOD_None = 0,
			METHOD_Nearest,
			METHOD_InverseLinear,
			METHOD_InverseSquare
			};

		//! Constructor.
		FILTER_GAPFILL (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			METHOD Method,								//!< Filling method
			INT32 GapWidth								//!< Gap width in cells (1-15)
			);

		// Destructor.
		virtual ~FILTER_GAPFILL ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class TESTPOS {
			public:
				TESTPOS () : m_dx(0), m_dy(0) { }
				TESTPOS (INT32 dx, INT32 dy) : m_dx(static_cast<INT8>(dx)), m_dy(static_cast<INT8>(dy)) { }
				INT8 GetDx () const { return (m_dx); }
				INT8 GetDy () const { return (m_dy); }
			private:
				INT8 m_dx;
				INT8 m_dy;
			};
		typedef MILIST<TESTPOS> TESTPOSLIST;

		typedef UINT8* PTRUINT8;

		METHOD m_Method;
		INT32 m_GapWidth;
		INT32 m_GapWidth21;
		DATATYPE m_DataType;
		std::vector<TESTPOSLIST> m_TestPosLists[8];	// Array of vectors of test positions, indexed by octant and distance
		PTRUINT8 *m_SrcValidityArray;
		DOUBLE_ARRAY<double> m_TotValue;
		SIMPLE_ARRAY<UINT16> m_UsedArray;
		DOUBLE_ARRAY<double> m_Weight;

		void AddTestItem (INT32 d, INT32 dx, INT32 dy);
		void FillCellUINT8 (INT32 row, INT32 col, BUFFER& TgtBuffer, const BUFFER& SrcBuffer);
		void FillCellSINT8 (INT32 row, INT32 col, BUFFER& TgtBuffer, const BUFFER& SrcBuffer);
		void FillCellUINT16 (INT32 row, INT32 col, BUFFER& TgtBuffer, const BUFFER& SrcBuffer);
		void FillCellSINT16 (INT32 row, INT32 col, BUFFER& TgtBuffer, const BUFFER& SrcBuffer);
		void FillCellUINT32 (INT32 row, INT32 col, BUFFER& TgtBuffer, const BUFFER& SrcBuffer);
		void FillCellSINT32 (INT32 row, INT32 col, BUFFER& TgtBuffer, const BUFFER& SrcBuffer);
		void FillCellFLOAT32 (INT32 row, INT32 col, BUFFER& TgtBuffer, const BUFFER& SrcBuffer);
		void FillCellFLOAT64 (INT32 row, INT32 col, BUFFER& TgtBuffer, const BUFFER& SrcBuffer);

		// BASE/STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_GAPFILL (const FILTER_GAPFILL&);
		FILTER_GAPFILL& operator= (const FILTER_GAPFILL&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERGAPFILL_H

