/**
 * \file spatmod/imagefiltermaskvalidity.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_MASKVALIDITY definition.
 *
 * \if NODOC
 * $Id: imagefiltermaskvalidity.h_v 1.1 2006/11/06 17:38:04 mju Exp $
 *
 * $Log: imagefiltermaskvalidity.h_v $
 * Revision 1.1  2006/11/06 17:38:04  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERMASKVALIDITY_H
#define	INC_SPATMOD_IMAGEFILTERMASKVALIDITY_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to apply mask to image "validity".
//! Both source and mask images must have the same dimensions.
//! The mask image must have a single sample per pixel.
//! The georeference from the first source that has any will be used.
//! Invalid or zero mask pixels will result in invalid result pixels, otherwise
//! validity will be left unchanged.
class FILTER_MASKVALIDITY : public FILTER {
	public:

		//! Constructor.
		FILTER_MASKVALIDITY (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnershipSource,				//!< Filter takes ownership of source image
			STAGE *pMaskImage,						//!< Mask, must have single component sample
			bool TakeOwnershipMask					//!< Filter takes ownership of mask stage and will delete when needed
			);

		// Destructor.
		virtual ~FILTER_MASKVALIDITY ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef FILTER BASECLASS;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_MASKVALIDITY (const FILTER_MASKVALIDITY&);
		FILTER_MASKVALIDITY& operator= (const FILTER_MASKVALIDITY&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERMASKVALIDITY_H

