/**
 * \file imagefilternulltoalpha.h <spatmod/imagefilternulltoalpha.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_NULLTOALPHA definition.
 *
 * \if NODOC
 * $Id: imagefilternulltoalpha.h_v 1.1 2007/07/11 16:47:00 mju Exp $
 *
 * $Log: imagefilternulltoalpha.h_v $
 * Revision 1.1  2007/07/11 16:47:00  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERNULLTOALPHA_H
#define	INC_SPATMOD_IMAGEFILTERNULLTOALPHA_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to apply/transfer null pixels to alpha.
class FILTER_NULLTOALPHA : public FILTER {
	public:

		//! Constructor.
		FILTER_NULLTOALPHA (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			bool MakeValid = true					//!< Make replaced values "valid"
			);

		// Destructor.
		virtual ~FILTER_NULLTOALPHA ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT16 m_AlphaSampleIdx;
		bool m_SrcHasAlpha;
		UINT8 m_NewValidity;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_NULLTOALPHA (const FILTER_NULLTOALPHA&);
		FILTER_NULLTOALPHA& operator= (const FILTER_NULLTOALPHA&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERNULLTOALPHA_H

