/**
 * \file imagesourcemgd.h <spatmod/imagesourcemgd.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_MGD definition.
 *
 * \if NODOC
 * $Id: imagesourcemgd.h_v 1.6 2007/01/25 20:33:08 dwilliss Exp $
 *
 * $Log: imagesourcemgd.h_v $
 * Revision 1.6  2007/01/25 20:33:08  dwilliss
 * Added ctor taking device as pointer with option to "take ownership"
 *
 * Revision 1.5  2006/11/01 23:10:20  mju
 * queryIterator.
 *
 * Revision 1.4  2006/04/11 21:55:54  mju
 * Add useXasAlpha for ctor.
 *
 * Revision 1.3  2005/12/16 16:36:51  mju
 * Declare trans2d_mapgen.
 *
 * Revision 1.2  2005/12/09 21:34:48  mju
 * Add optional transformation in ctor.
 *
 * Revision 1.1  2005/12/09 21:26:56  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEMGD_H
#define	INC_SPATMOD_IMAGESOURCEMGD_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

namespace MGD {
	class DEVICE;
	}
class TRANS2D_MAPGEN;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Source which supports reading from MGD device.
class SOURCE_MGD : public SOURCE {
	public:

		//! Construct and attach to specified device.
		//! Device must support MGD::CAPABILITY_GetImage or error will be returned by Initialize.
		explicit SOURCE_MGD (
			MGD::DEVICE& Device,							//!< Device to attach
			const TRANS2D_MAPGEN *pTransToCRS = 0,	//!< Transformation from device to map CRS
			bool UseXasAlpha = false					//!< Treat RGBX and similar as RGBA
			);

		//! Construct and attach to specified device.
		//! Device must support MGD::CAPABILITY_GetImage or error will be returned by Initialize.
		explicit SOURCE_MGD (
			MGD::DEVICE* Device,							//!< Device to attach
			bool bTakeOwnership,							//!< If true, SOURCE_MGD will "own" the device and delete in its own destructor
			const TRANS2D_MAPGEN *pTransToCRS = 0,	//!< Transformation from device to map CRS
			bool UseXasAlpha = false					//!< Treat RGBX and similar as RGBA
			);

		// Destructor.
		virtual ~SOURCE_MGD ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGD::DEVICE& m_Device;
		MGD::DEVICE* m_DeleteThis;
		bool m_UseXasAlpha;
		int m_SampleOffset[4];

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_MGD (const SOURCE_MGD&);
		SOURCE_MGD& operator= (const SOURCE_MGD&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEMGD_H

