/**
 * \file spatmod/imagesourcervchyper.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_RVC_HYPER definition.
 *
 * \if NODOC
 * $Id: imagesourcervchyper.h_v 1.2 2011/10/19 18:40:48 mju Exp $
 *
 * $Log: imagesourcervchyper.h_v $
 * Revision 1.2  2011/10/19 18:40:48  mju
 * Remove redundant hdrs.
 *
 * Revision 1.1  2006/11/03 16:48:07  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCERVCHYPER_H
#define	INC_SPATMOD_IMAGESOURCERVCHYPER_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef	INC_RVC_HYPSPECT_H
#include <rvc/hypspect.h>
#endif

#ifndef	INC_RVC_NULLMASK_H
#include <rvc/nullmask.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! RVC Hyperspectral Image source.
class SOURCE_RVC_HYPER : public SOURCE {
	public:

		//! Construct with optional band selection.
		SOURCE_RVC_HYPER (
			const RVC::OBJITEM& ObjItem,						//!< Reference to object location
			const UINT16 *Bands = 0,							//!< Band index array
			UINT16 NumBands = 0,									//!< Number of bands to select
			PIXEL::TYPE PixelType = PIXEL::TYPE_Invalid	//!< Pixel type override
			);

		//! Destructor.
		virtual ~SOURCE_RVC_HYPER ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL::TYPE m_PixelType;
		RVC::IMAGE::CONVMODE m_ConvMode;
		RVC::OBJITEM m_ObjItemMain;
		mutable RVC::HYPERSPECTRAL m_ObjImage;
		mutable RVC::NULLMASK m_ObjMask;
		bool m_DoingPull;
		SIMPLE_ARRAY<UINT16> m_SampleBands;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_ComputeSampleStats (STAT_UNIVARIATE *SampleStatsArray, STATSVALIDITY StatsValidity);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_RVC_HYPER (const SOURCE_RVC_HYPER&);
		SOURCE_RVC_HYPER& operator= (const SOURCE_RVC_HYPER&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCERVCHYPER_H

