/**
 * \file spatmod/imagesourcetiff.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_TIFF definition.
 *
 * \if NODOC
 * $Id: imagesourcetiff.h_v 1.3 2011/10/11 14:46:33 mju Exp $
 *
 * $Log: imagesourcetiff.h_v $
 * Revision 1.3  2011/10/11 14:46:33  mju
 * Remove unneeded hdr.
 *
 * Revision 1.1  2007/06/20 19:34:07  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCETIFF_H
#define	INC_SPATMOD_IMAGESOURCETIFF_H

#ifndef	INC_SPATMOD_IMAGESOURCEFILE_H
#include <spatmod/imagesourcefile.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
class TIFFEXT;
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! TIFF pipeline source.
class SOURCE_TIFF : public SOURCE_FILE {
	public:

		//! Construct with FILEPATH.
		explicit SOURCE_TIFF (
			const FILEPATH& FilePath
			);

		//! Construct with memory buffer (SIMPLE_ARRAY).
		//! Will take ownership of buffer.
		explicit SOURCE_TIFF (
			SIMPLE_ARRAY<UINT8>& MemBuffer	//!< Memory buffer, will be returned empty
			);

		//! Construct with memory buffer (pointer/size).
		//! Will take ownership of buffer, pointer will be returned as NULL
		SOURCE_TIFF (
			UINT8*& MemBuffer,					//!< Memory buffer, will be returned as NULL
			UINT32 BufferSize						//!< Buffer size
			);

		//! Destructor.
		virtual ~SOURCE_TIFF ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILEPATH m_FilePath;
		TIFFEXT *m_pReader;
		SIMPLE_ARRAY<UINT8> m_ReadBuffer;
		bool m_DoingPull;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual void v_GetMetadataThisOnly (MISTRING& result, METADATACONTENT content, METADATAFORMAT format) const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_TIFF (const SOURCE_TIFF&);
		SOURCE_TIFF& operator= (const SOURCE_TIFF&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCETIFF_H

