/**
 * \file spatmod/imagetargettilesetgooglemaps.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_TILESET_GOOGLEMAPS definition.
 *
 * \if NODOC
 * $Id: imagetargettilesetgooglemaps.h_v 1.24 2012/03/14 22:00:02 bcolombe Exp $
 *
 * $Log: imagetargettilesetgooglemaps.h_v $
 * Revision 1.24  2012/03/14 22:00:02  bcolombe
 * Changes for 24,32 bit terrain
 *
 * Revision 1.23  2012/01/26 22:31:21  bcolombe
 * Remove IsTerrain, Add General Filter Delegates
 *
 * Revision 1.22  2012/01/18 15:32:44  bcolombe
 * Add IsTerrain
 *
 * Revision 1.21  2012/01/11 23:27:53  bcolombe
 * Add SetCreteHTML
 *
 * Revision 1.20  2011/10/11 15:01:45  mju
 * Remove unneeded hdr.
 *
 * Revision 1.19  2011/04/20 23:21:48  bcolombe
 * Added ResampleMethod for reduced levels
 *
 * Revision 1.18  2011/04/14 16:42:25  bcolombe
 * Remove extents from ctor, determine from input
 *
 * Revision 1.17  2010/11/01 22:45:27  bcolombe
 * Imgformat changes
 *
 * Revision 1.16  2010/06/03 22:48:39  bcolombe
 * Added UsePalette to ComputeReduced
 *
 * Revision 1.15  2010/05/18 16:36:21  bcolombe
 * Add ComputeReducedLevel that takes source & target locations
 *
 * Revision 1.14  2010/04/19 20:34:55  bcolombe
 * Add bZipOutput to ComputeReducedLevel
 *
 * Revision 1.13  2010/04/19 16:15:32  bcolombe
 * Add bOverwriteExisting to CreateHTML
 * Add PHPLocation to CreateHTML
 *
 * Revision 1.12  2010/04/07 22:44:07  bcolombe
 * *** empty log message ***
 *
 * Revision 1.11  2010/04/07 17:55:12  bcolombe
 * *** empty log message ***
 *
 * Revision 1.10  2010/04/07 17:36:20  bcolombe
 * Added ZipLevel
 *
 * Revision 1.9  2010/04/02 22:25:03  bcolombe
 * Make CreateHTML and ComputeReducedLevel static
 * Added GetFormat
 *
 * Revision 1.8  2010/03/31 22:38:24  bcolombe
 * Added MinLevel to CreateHTML
 *
 * Revision 1.7  2010/03/18 16:04:40  bcolombe
 * Chnage to constructor
 *
 * Revision 1.6  2009/12/03 21:55:54  bcolombe
 * Added optional message to ComputeReducedLevel
 *
 * Revision 1.5  2009/11/04 23:47:15  bcolombe
 * Constructor takes basepath instead of htmlpath
 *
 * Revision 1.4  2009/11/04 21:29:19  bcolombe
 * Changed description of filepath in constructor
 *
 * Revision 1.3  2009/09/08 15:17:45  bcolombe
 * Make ComputeReducedLevel public
 *
 * Revision 1.2  2009/08/13 17:44:22  mju
 * Add m_pstatusContext member.
 *
 * Revision 1.1  2009/08/06 17:01:45  mju
 * Initial revision
 *
 * Revision 1.15  2009/07/24 23:04:28  bcolombe
 * Make changes for TILESET class
 *
 * Revision 1.12  2009/06/17 22:12:27  bcolombe
 * Added changes for merge tileset
 *
 * Revision 1.8  2009/05/08 17:15:04  mju
 * In computeTileValues add "isBottomRight" to handle last pixel properly.
 *
 * Revision 1.1  2009/02/27 20:35:07  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETTILESETGOOGLEMAPS_H
#define	INC_SPATMOD_IMAGETARGETTILESETGOOGLEMAPS_H

#ifndef	INC_SPATMOD_IMAGETARGETTILESETFILE_H
#include <spatmod/imagetargettilesetfile.h>
#endif

class MISTATUSCONTEXT;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Google Maps target.
class TARGET_TILESET_GOOGLEMAPS : public TARGET_TILESET_FILE {
	public:
		//! Create GoogleMaps HTML file
		static ERRVALUE CreateHTML (
			TILESET& Tileset,								//!< Tileset to use
			bool bOverwriteExisting = true,					//!< Write over any existing files
			INT32 MinLevel = -1,							//!< Minimum zoom level to include
			MISTRING GoogleMapsKey = TEXTID__None,			//!< Google maps key
			MISTRING NoDataText = TEXTID__None,				//!< No data text
			MISTRING PHPLocation = TEXTID__None				//!< Define location to look for PHP script
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			INT32 level,										//!< Level to compute
			MISTRING message = TEXTID_Creating_ResolutionLevelN	//!< Context message to display
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			const FILEPATH& SourcePath,							//!< Location of source tiles
			const FILEPATH& TargetPath,							//!< Location to output tiles
			INT32 level,										//!< Level to compute
			MISTRING message = TEXTID_Creating_ResolutionLevelN	//!< Context message to display
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			INT32 level,										//!< Level to compute
			TILESET::IMGFORMAT ImgFormat = TILESET::IMGFORMAT_Invalid,	//!< ImgFormat
			UINT16 QualityJPEG = 75,							//!< Quality of JPEG output
			MISTRING message = TEXTID_Creating_ResolutionLevelN,//!< Context message to display
			bool bZipOutput = false								//!< Zip results
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			const FILEPATH& SourcePath,							//!< Location of source tiles
			const FILEPATH& TargetPath,							//!< Location to output tiles
			INT32 level,										//!< Level to compute
			TILESET::IMGFORMAT ImgFormat,						//!< ImgFormat
			const IMAGE::PIXEL& PixelProperties,				//!< Pixel Properties
			UINT16 QualityJPEG = 75,							//!< Quality of JPEG output
			MISTRING message = TEXTID_Creating_ResolutionLevelN,//!< Context message to display
			bool bZipOutput = false,								//!< Zip results
			RESAMPLEMETHOD ResampleMethod = RESAMPLEMETHOD_Nearest	//!< Resample Method to use for zoom filter
			);

		//! Construct target.
		TARGET_TILESET_GOOGLEMAPS (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			TILESET::IMGFORMAT ImgFormat,				//!< Image tile file format
			const FILEPATH& filepath,					//!< Basepath used for creating TSD and HTML files
			INT32 TileSize,								//!< Tile Size (256 or 128)
			INT32 MinLevel,								//!< Minimum zoom level to compute
			INT32 MaxLevel								//!< Maximum zoom level to compute
			);

		//! Destructor.
		virtual ~TARGET_TILESET_GOOGLEMAPS ();

		//! Set google maps key used in html produced
		void SetGoogleMapsKey (
			const MISTRING& key
			) { m_GoogleMapsKey = key; }

		//! Set text to display in empty tiles
		void SetNoDataText (
			const MISTRING& text
			) { m_NoDataText = text; }

		//! Set parameters specific to JPEG format.
		void SetParmsJPEG (
			INT16 quality
			) { m_QualityJPEG = quality; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class IMAGEGOOGLEMAPS;

		UINT16 m_QualityJPEG;
		FILEPATH m_basepath;
		MISTATUSCONTEXT *m_pStatusContext;

		// For html generation:
		MISTRING m_GoogleMapsKey;
		MISTRING m_NoDataText;

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();

		// TARGET_TILESET_FILE overrides.
		ERRVALUE v_CreateTargetFile (IMAGE::STAGE *pStageInput, const FILEPATH& filepath, const IMAGE::FORMAT& format, IMAGE::TARGET_FILE *& pTargetFile) const;

		// Unimplemented
		TARGET_TILESET_GOOGLEMAPS (const TARGET_TILESET_GOOGLEMAPS&);
		TARGET_TILESET_GOOGLEMAPS& operator= (const TARGET_TILESET_GOOGLEMAPS&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETTILESETGOOGLEMAPS_H

